@@ -169,10 +169,13 @@ init (GnomePilotCapplet *gpcap)
 	priv->state = dupPilotState (priv->orig_state);
 
 	/* Gui stuff */
-	priv->xml = glade_xml_new (GLADEDATADIR "/gpilotd-capplet.glade", "CappletMain", NULL);
+	priv->xml = glade_xml_new ("gpilotd-capplet.glade", "CappletMain", NULL);
 	if (!priv->xml) {
-		g_message ("gnome-pilot-capplet init(): Could not load the Glade XML file!");
-		goto error;
+		priv->xml = glade_xml_new (GLADEDATADIR "/gpilotd-capplet.glade", "CappletMain", NULL);
+		if (!priv->xml) {
+			g_message ("gnome-pilot-capplet init(): Could not load the Glade XML file!");
+			goto error;
+		}
 	}
 	
 	if (!get_widgets (gpcap)) {
