@@ -46,6 +46,7 @@ gboolean check_editable (GtkEditable *editable);
 /* General routines */
 GPilotPilot *get_default_pilot (PilotState *state);
 GPilotDevice *get_default_device (PilotState *state);
+const char *get_default_pilot_charset(void);
 
 gchar *next_cradle_name (PilotState *state);
 gchar *next_pilot_name (PilotState *state);
@@ -57,6 +58,7 @@ const gchar* display_sync_type_name (gboolean enabled, GnomePilotConduitSyncType
 gboolean check_pilot_info (GPilotPilot* pilot1, GPilotPilot *pilot2);
 gboolean check_device_info (GPilotDevice* device1, GPilotDevice *device2);
 gboolean check_base_directory (const gchar *dir_name);
+gboolean check_pilot_charset (const gchar *charset);
 
 /* Dialogs */
 void error_dialog (GtkWindow *parent, gchar *mesg, ...);
