@@ -21,7 +21,7 @@
  *          Robert Mibus
  */
 
-/* $Id: backup_conduit.c,v 1.55 2006/06/18 22:24:55 mcdavey Exp $ */
+/* $Id: backup_conduit.c 1524 2008-01-03 09:12:45Z mcdavey $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -473,7 +473,10 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 	if (err < 0) {
 		g_warning ("error (%s) in opening '%s'", dlp_strerror (err), PI_DBINFO (dbinfo)->name);
 		result = -1;
-		goto db_close;
+		/* DB not successfully opened, so don't try and close (caused
+		 * error on Tungsten E2) probably cause of Ubuntu bug #81396
+		 */
+		goto exit_and_free; 
 	}
 
 	err = dlp_ReadOpenDBInfo (dbinfo->pilot_socket, 
@@ -845,6 +848,7 @@ static GtkWidget
 	GtkWidget *entry, *label;
 	GtkWidget *button, *spin;
 	GtkObject *adjustment;
+	AtkObject *atk_widget;
 
 	vbox = gtk_vbox_new(FALSE, GNOME_PAD);
 
@@ -853,7 +857,7 @@ static GtkWidget
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, GNOME_PAD);
 
-	label = gtk_label_new(_("Backup directory:"));
+	label = gtk_label_new_with_mnemonic(_("_Backup directory:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
     
@@ -864,7 +868,11 @@ static GtkWidget
 			   GTK_SIGNAL_FUNC(insert_dir_callback),
 			   NULL);
 
-	label = gtk_label_new(_("Only backup changed bases:"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), entry);
+	atk_widget = gtk_widget_get_accessible (entry);
+	atk_object_set_name (atk_widget, _("Backup directory"));
+
+	label = gtk_label_new_with_mnemonic (_("O_nly backup changed bases"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1,2);
 
@@ -872,7 +880,11 @@ static GtkWidget
 	gtk_object_set_data(GTK_OBJECT(vbox), "only_changed", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1,2);
 
-	label = gtk_label_new(_("Remove local base if deleted on PDA:"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), button);
+	atk_widget = gtk_widget_get_accessible (button);
+	atk_object_set_name (atk_widget, _("Only backup changed bases"));
+
+	label = gtk_label_new_with_mnemonic(_("_Remove local base if deleted on pilot"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2,3);
 
@@ -880,7 +892,11 @@ static GtkWidget
 	gtk_object_set_data(GTK_OBJECT(vbox), "remove_local", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2,3);
 
-	label = gtk_label_new(_("# of old backups to keep:"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), button);
+	atk_widget = gtk_widget_get_accessible (button);
+	atk_object_set_name (atk_widget, _("Remove local base if deleted on pilot"));
+
+	label = gtk_label_new_with_mnemonic (_("# of old backups to _keep"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3,4);
 
@@ -889,6 +905,10 @@ static GtkWidget
 	gtk_object_set_data(GTK_OBJECT(vbox), "no_of_backups", adjustment);
 	gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 3, 4);
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), spin);
+	atk_widget = gtk_widget_get_accessible (spin);
+	atk_object_set_name (atk_widget, _("# of old backups to keep"));
+
 	return vbox;
 }
 
