@@ -23,6 +23,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <gnome.h>
 #include <pi-source.h>
 #include <pi-socket.h>
@@ -31,6 +35,75 @@
 #include "gpmarshal.h"
 #include "manager.h"
 
+
+#ifdef PILOT_LINK_0_12
+/* Compatibility routines for old API in pilot-link-0.11 */
+extern int
+gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordById (int sd, int fHandle, recordid_t id, void *buffer,
+							    int *index, int *size, int *attr, int *category);
+
+extern int
+gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordByIndex (int sd, int fHandle, int index, void *buffer,
+							       recordid_t * id, int *size, int *attr,
+							       int *category);
+extern int
+gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadNextModifiedRec (int sd, int fHandle, void *buffer, recordid_t * id,
+								 int *index, int *size, int *attr, int *category);
+
+
+int
+gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordById (int sd, int fHandle, recordid_t id, void *buffer,
+							    int *index, int *size, int *attr, int *category)
+{
+	pi_buffer_t *pi_buf = pi_buffer_new (0xffff);
+	int result = dlp_ReadRecordById (sd, fHandle, id, 
+					 pi_buf,
+					 index, attr, category);
+	memcpy (buffer, pi_buf->data, pi_buf->used);
+	if (NULL!=size) {
+		*size = pi_buf->used;
+	}
+	pi_buffer_free (pi_buf);
+
+	return result;
+}
+
+int
+gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordByIndex (int sd, int fHandle, int index, void *buffer,
+							       recordid_t * id, int *size, int *attr,
+							       int *category)
+{
+	pi_buffer_t *pi_buf = pi_buffer_new (0xffff);
+	int result = dlp_ReadRecordByIndex (sd, fHandle, index, 
+					    pi_buf,
+					    id, attr, category);
+	memcpy (buffer, pi_buf->data, pi_buf->used);
+	if (NULL!=size) {
+		*size = pi_buf->used;
+	}
+	pi_buffer_free (pi_buf);
+
+	return result;
+}
+
+int
+gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadNextModifiedRec (int sd, int fHandle, void *buffer, recordid_t * id,
+								 int *index, int *size, int *attr, int *category)
+{
+	pi_buffer_t *pi_buf = pi_buffer_new (0xffff);
+	int result = dlp_ReadNextModifiedRec (sd, fHandle, 
+					      pi_buf, 
+					      id, index, attr, category);
+	memcpy (buffer, pi_buf->data, pi_buf->used);
+	if (NULL!=size) {
+		*size = pi_buf->used;
+	}
+	pi_buffer_free (pi_buf);
+
+	return result;
+}
+#endif
+
 enum {
 	MATCH_RECORD,
 	FREE_MATCH,
@@ -60,13 +133,13 @@ typedef enum StandardAbsSyncDirection StandardAbsSyncDirection;
 #define SyncBothWays SyncToRemote|SyncToLocal
 
 
-#define LOG_CASE5 _("deleted pilot record modified locally, not deleting\n")
-#define LOG_CASE6 _("deleted local record modified on pilot, not deleting\n")
-#define LOG_CASE10 _("merge conflict, pilot and local record swapped\n")
-#define LOG_CASE13 _("archive and change conflict, pilot and local record swapped\n")
-#define LOG_CASE15 _("archive and change conflict, local record sent to pilot\n")
-#define LOG_CASE18 _("archive and change conflict, pilot record saved locally\n")
-#define LOG_CASE20 _("merge conflict, pilot and local record swapped\n")
+#define LOG_CASE5 _("deleted PDA record modified locally, not deleting\n")
+#define LOG_CASE6 _("deleted local record modified on PDA, not deleting\n")
+#define LOG_CASE10 _("merge conflict, PDA and local record swapped\n")
+#define LOG_CASE13 _("archive and change conflict, PDA and local record swapped\n")
+#define LOG_CASE15 _("archive and change conflict, local record sent to PDA\n")
+#define LOG_CASE18 _("archive and change conflict, PDA record saved locally\n")
+#define LOG_CASE20 _("merge conflict, PDA and local record swapped\n")
 
 #define Result(v) result = v; break
 
@@ -360,7 +433,7 @@ gnome_pilot_conduit_standard_real_copy_to_pilot     (GnomePilotConduitStandard *
 		
 		/* nuke all records */
 		if (dlp_DeleteRecord (dbinfo->pilot_socket, dbinfo->db_handle, 1, 0) < 0) {
-			g_warning(_("Unable to delete all records in pilot database, aborting operation."));
+			g_warning(_("Unable to delete all records in PDA database, aborting operation."));
 			Result(-4);
 		}
 		
@@ -433,7 +506,11 @@ gnome_pilot_conduit_standard_real_copy_from_pilot   (GnomePilotConduitStandard *
 		}
 		
 		/* copy all records */
+#ifdef PILOT_LINK_0_12
+		while (gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordByIndex (dbinfo->pilot_socket,
+#else
 		while (dlp_ReadRecordByIndex (dbinfo->pilot_socket,
+#endif
 					      dbinfo->db_handle,
 					      index,
 					      remote.record,
@@ -1081,7 +1158,11 @@ standard_abs_sync_record (GnomePilotConduitStandardAbs *conduit,
 		g_message("gpilotd: retrieve %ld from pilot",local->ID);
 		remote = g_new0(PilotRecord,1);
 		remote->record = malloc(0xffff);
+#ifdef PILOT_LINK_0_12
+		gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordById(handle,db,
+#else
 		dlp_ReadRecordById(handle,db,
+#endif
 				   local->ID,
 				   remote->record,
 				   &index,
@@ -1438,7 +1519,11 @@ standard_abs_merge_to_local (GnomePilotConduitStandardAbs *conduit,
 	retval = 0;
 	remote.record = buffer;
 
-	while (dlp_ReadRecordByIndex(handle,db,
+#ifdef PILOT_LINK_0_12
+	while (gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordByIndex (handle, db,
+#else
+	while (dlp_ReadRecordByIndex (handle, db,
+#endif
 				     index,
 				     remote.record,
 				     &remote.ID,
@@ -1477,7 +1562,11 @@ SlowSync (int handle,
 
 	g_message("Performing Slow Synchronization");
 
-	while (dlp_ReadRecordByIndex(handle,db,
+#ifdef PILOT_LINK_0_12
+	while (gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadRecordByIndex (handle, db,
+#else
+	while (dlp_ReadRecordByIndex (handle, db,
+#endif
 				     index,
 				     remote.record,
 				     &remote.ID,
@@ -1517,7 +1606,11 @@ FastSync (int handle,
 
 	g_message("Performing Fast Synchronization");
 
+#ifdef PILOT_LINK_0_12
+	while (gnome_pilot_compat_with_pilot_link_0_11_dlp_ReadNextModifiedRec (handle, db,
+#else
 	while (dlp_ReadNextModifiedRec (handle, db,
+#endif
 					remote.record,
 					&remote.ID,
 					&index,
