@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include "gnome-pilot-structures.h"
 #include "gpilot-gui.h"
 #include <libgnome/libgnome.h>
@@ -38,12 +39,11 @@
 #include <signal.h>
 
 /* From pi-csd */
-#include <signal.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#include <net/if.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <pi-source.h>
 
 #define LOCK_DIR "/var/lock"
 #define LOCK_BINARY 0
@@ -300,7 +300,7 @@ gpilot_serial_device_init (GPilotDevice *device)
 		g_free (device->port);
 		device->port = NULL;
 
-		close (device->fd);
+		pi_close (device->fd);
 		device->fd = 0;
 
 		device->io = NULL;
@@ -318,7 +318,7 @@ static void
 gpilot_serial_device_free (GPilotDevice *device)
 {
 	if (device->fd) {
-		close (device->fd);
+		pi_close (device->fd);
 	}
 	g_free (device->name);
 	g_free (device->port);
@@ -328,10 +328,12 @@ gpilot_serial_device_free (GPilotDevice *device)
 static void
 gpilot_serial_device_deinit (GPilotDevice *device)
 {
+	GError *myerr = NULL;
+
 	if (device->io) {
 		g_source_remove (device->in_handle);
 		g_source_remove (device->err_handle);
-		g_io_channel_close (device->io);
+		g_io_channel_shutdown (device->io, 0, &myerr);
 		g_io_channel_unref (device->io);
 	}
 }
@@ -380,154 +382,6 @@ gpilot_irda_device_deinit (GPilotDevice *device)
 
 #ifdef WITH_NETWORK
 
-/* 
- * pi-csd.c: Connection Service Daemon, required for accepting 
- *           logons via NetSync (tm)
- * Copyright (c) 1997, Kenneth Albanowski
- *
- * While this function is useful in pi-csd, it is intended also to
- * be a demonstration of the proper (or improper, if I'm unlucky) techniques
- * to retrieve networking information.
- */
-static void
-fetch_host (char *hostname, int hostlen, struct in_addr *address,
-	   struct in_addr *mask)
-{
-#if defined (SIOCGIFCONF) && defined (SIOCGIFFLAGS)
-	int s, n, i;
-	struct ifconf ifc;
-	struct ifreq *ifr, ifreqaddr, ifreqmask;
-#endif
-	struct hostent *hent;
-
-#ifdef HAVE_GETHOSTNAME
-	/* Get host name the easy way */
-
-	gethostname (hostname, hostlen);
-#else
-# ifdef HAVE_UNAME
-	struct utsname uts;
-
-	if (uname (&uts) == 0) {
-		strncpy (hostname, uts.nodename, hostlen - 1);
-		hostname[hostlen - 1] = '\0';
-	}
-# endif	/*def HAVE_UNAME */
-#endif /*def HAVE_GETHOSTNAME */
-
-	/* Get host address through DNS */
-	hent = gethostbyname (hostname);
-
-	if (hent) {
-		while (*hent->h_addr_list) {
-			struct in_addr haddr;
-
-			memcpy (&haddr, *(hent->h_addr_list++), sizeof (haddr));
-			if (haddr.s_addr != inet_addr ("127.0.0.1"))
-				memcpy (address, &haddr, sizeof (haddr));
-		}
-	}
-#if defined (SIOCGIFCONF) && defined (SIOCGIFFLAGS)
-	s = socket (AF_INET, SOCK_DGRAM, 0);
-
-	if (s < 0)
-		return;
-
-	ifc.ifc_buf = calloc (1024, 1);
-	ifc.ifc_len = 1024;
-
-	if (ioctl (s, SIOCGIFCONF, (char *) &ifc) < 0)
-		goto done;
-
-	n = ifc.ifc_len;
-	for (i = 0; i < n; i += ifreq_size (*ifr)) {
-		struct sockaddr_in *a;
-		struct sockaddr_in *b;
-
-		ifr = (struct ifreq *) ((caddr_t) ifc.ifc_buf + i);
-		a = (struct sockaddr_in *) &ifr->ifr_addr;
-		strncpy (ifreqaddr.ifr_name, ifr->ifr_name, sizeof (ifreqaddr.ifr_name));
-		strncpy (ifreqmask.ifr_name, ifr->ifr_name, sizeof (ifreqmask.ifr_name));
-
-		if (ioctl (s, SIOCGIFFLAGS, (char *) &ifreqaddr) < 0)
-			continue;
-
-		/* Reject loopback device */
-#ifdef IFF_LOOPBACK
-		if (ifreqaddr.ifr_flags & IFF_LOOPBACK)
-			continue;
-#endif /*def IFF_LOOPBACK */
-
-#ifdef IFF_UP
-		/* Reject down devices */
-		if (!(ifreqaddr.ifr_flags & IFF_UP))
-			continue;
-#endif /*def IFF_UP */
-
-		if (ifr->ifr_addr.sa_family != AF_INET)
-			continue;
-
-		/* If it is a point-to-point device, use the dest address */
-#if defined (IFF_POINTOPOINT) && defined (SIOCGIFDSTADDR)
-		if (ifreqaddr.ifr_flags & IFF_POINTOPOINT) {
-			if (ioctl (s, SIOCGIFDSTADDR, (char *) &ifreqaddr) < 0)
-				break;
-
-			a = (struct sockaddr_in *) &ifreqaddr.ifr_dstaddr;
-
-			if (address->s_addr == 0) {
-				memcpy (address, &a->sin_addr, sizeof (struct in_addr));
-			}
-		}
-		else
-#endif /*defined (IFF_POINTOPOINT) && defined (SIOCGIFDSTADDR) */
-			/* If it isn't a point-to-point device, use the address */
-#ifdef SIOCGIFADDR
-		{
-			if (ioctl (s, SIOCGIFADDR, (char *) &ifreqaddr) < 0)
-				break;
-
-			a = (struct sockaddr_in *) &ifreqaddr.ifr_addr;
-
-			if (address->s_addr == 0) {
-				memcpy (address, &a->sin_addr, sizeof (struct in_addr));
-			}
-		}
-#endif /*def SIOCGIFADDR */
-		/* OK, we've got an address */
-
-		/* Compare netmask against the current address and see if it seems to match. */
-#ifdef SIOCGIFNETMASK
-		if (ioctl (s, SIOCGIFNETMASK, (char *) &ifreqmask) < 0)
-			break;
-
-/* Is there any system where we need to use ifr_netmask?  */
-#if 1
-		b = (struct sockaddr_in *) &ifreqmask.ifr_addr;
-#else
-		b = (struct sockaddr_in *) &ifreqmask.ifr_netmask;
-#endif
-
-		if ((mask->s_addr == 0) && (address->s_addr != 0)) {
-			if ((b->sin_addr.s_addr & a->sin_addr.s_addr) ==
-			    (b->sin_addr.s_addr & address->s_addr)) {
-				memcpy (mask, &b->sin_addr, sizeof (struct in_addr));
-
-				/* OK, we've got a netmask */
-
-				break;
-			}
-		}
-#endif /*def SIOCGIFNETMASK */
-
-	}
-
- done:
-	free (ifc.ifc_buf);
-	close (s);
-#endif /*defined (SIOCGIFCONF) && defined (SIOCGIFFLAGS) */
-}
-
 /* gpilot_network_device_init (GPilotDevice *device)
  * pi-csd.c: Connection Service Daemon, required for accepting 
  *           logons via NetSync (tm)
@@ -537,85 +391,91 @@ fetch_host (char *hostname, int hostlen, struct in_addr *address,
 static gint 
 gpilot_network_device_init (GPilotDevice *device)
 {
-/*
- * Open a file descriptor. device->fd
- * Check fd. return -1 if fail.
- * device->io = g_io_channel_unix_new (device->fd);
- * g_io_channel_ref (device->io);
- * return 0;
- *
- */
+#ifndef PILOT_LINK_0_12
+	struct pi_sockaddr addr;
+#endif
+#define MAXLINELEN 256
+	char line[MAXLINELEN];
+	char pi_net[100];
+	int ret, fd;
+	static gboolean bluetooth_warning_done = FALSE;
 
-/* char hostname[130];	this is device->host */
-	struct in_addr address, netmask;
+	memset(pi_net, 0, sizeof(pi_net));
 
-
-/*   int sockfd;	this is device->fd */
-	struct sockaddr_in serv_addr;
-
-	struct hostent *hent;
-/*   int quiet = 0;		It's Always quiet here */
-
-	memset (&address, 0, sizeof (address));
-	memset (&netmask, 0, sizeof (netmask));
-
-/*   hostname[0] = 0; */
-/* device->host = g_new (gchar, 130);
-   g_free (device->host);
-*/
-
-/* This is used if the hostname is not correct in the conf file
-   I'll move it to the conf file saving later */
-	fetch_host (device->host, 128, &address, &netmask); 
-	
-/* device->host I already have from the config file */
-
-	if (inet_aton (device->ip, &address) == 0) /* Invalid */
-	{
-		if ((hent = gethostbyname (device->ip))) {
-			memcpy (&address.s_addr, hent->h_addr, sizeof (address));
+	if (device->type == PILOT_DEVICE_BLUETOOTH) {
+		strncpy(pi_net, "bt:", 3);
+	} else {
+		g_assert (device->type == PILOT_DEVICE_NETWORK);
+		strncpy(pi_net, "net:", 4);
+		if (device->ip != NULL && (*device->ip != 0x0)) {
+			struct 	sockaddr_in serv_addr;
+			/* Verify the IP address is valid */
+			memset(&serv_addr, 0, sizeof(serv_addr));
+			serv_addr.sin_family = AF_INET;
+			serv_addr.sin_addr.s_addr = inet_addr(device->ip);
+			if (serv_addr.sin_addr.s_addr == (in_addr_t)-1) {
+				struct hostent *hostent = gethostbyname(device->ip);
+				if (!hostent) {
+					g_warning ("Device [%s]: Bad IP address/hostname: %s",
+					    device->name, device->ip);
+					return -1;
+				}
+			}
+			strncat(pi_net, device->ip, sizeof(pi_net) - 2 - strlen(pi_net));
 		} else {
-			g_message ("Invalid ip address '%s'", device->ip);
-			return -1;
+			strncat(pi_net, "any", 3);
 		}
 	}
-
-	if (inet_aton (device->netmask, &netmask) == 0) {
-		g_message ("Invalid netmask '%s'", device->netmask);
-		return -1;
-	}
-
-	/* cannot execute without address and hostname */
-	if ((address.s_addr == 0) || (strlen (device->host) == 0))
-	{
-		g_message ("Cannot execute without ip address and hostname.");
-		return -1;
-	}
-
-	device->fd = socket (AF_INET, SOCK_DGRAM, 0);
+#ifdef PILOT_LINK_0_12
+	device->fd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_DLP);
+#else
+	device->fd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_NET);
+#endif
 	if (device->fd < 0) {
-		g_message ("Unable to get socket");
+		g_warning ("Device [%s, %s]: Unable to get socket: %s", 
+		    device->name, pi_net, strerror(errno));
 		return -1;
 	}
 
-	memset (&serv_addr, 0, sizeof (serv_addr));
-	serv_addr.sin_family = AF_INET;
-	serv_addr.sin_addr.s_addr = htonl (INADDR_ANY);
-	serv_addr.sin_port = htons (14237);
+#ifdef PILOT_LINK_0_12
+	ret = pi_bind (device->fd, pi_net);
+#else
+	strncpy (addr.pi_device,pi_net, strlen(pi_net));
 
-	if (bind (device->fd, (struct sockaddr *) &serv_addr, sizeof (serv_addr)) < 0) {
-		g_message ("Unable to bind socket");
+	ret = pi_bind (device->fd, (struct sockaddr*)&addr, sizeof (addr));
+#endif
+	if (ret < 0) {
+		if (device->type == PILOT_DEVICE_BLUETOOTH) {
+			g_snprintf(line, MAXLINELEN,_("Bluetooth Device [%s]: Unable to "
+				"bind socket: err %d (check pilot-link was compiled "
+				"with bluetooth support)"),
+			    device->name, ret);
+			/* only give GUI warning once... */
+			if (!bluetooth_warning_done) {
+				gpilot_gui_warning_dialog(line);
+				bluetooth_warning_done = TRUE;
+			}
+		} else {
+			g_snprintf(line, MAXLINELEN,_("Device [%s, %s]: Unable to "
+				"bind socket: err %d"),
+			    device->name, pi_net, ret);
+		}
+		g_warning(line);
 		return -1;
 	}
-	/*
-	g_message ("Connection Service Daemon for Palm Computing (tm) device active.");
-	g_message ("Accepting connection requests for '%s' at %s with mask %s", 
-		   device->host, inet_ntoa (address), inet_ntoa (netmask));
-	*/
 
-	/* We don't want to infinite loops here */
-	/* Lets try something else instead */
-	device->io = g_io_channel_unix_new (device->fd);
+	/* Now listen for incoming connections */
+	if (pi_listen (device->fd, 1) != 0) {
+		g_warning ("Device [%s, %s]: Error from listen: %s",
+		    device->name, pi_net, strerror (errno));
+		pi_close(device->fd);
+		return -1;
+	}
+	/* Register an interest in the socket, to get events when devices connect */
+	fd = dup(device->fd);
+	fcntl(fd, F_SETFD, FD_CLOEXEC); /* don't let bonobo keep references alive */
+	fcntl(device->fd, F_SETFD, FD_CLOEXEC);
+	device->io = g_io_channel_unix_new (fd);
 	g_io_channel_ref (device->io);
 
 	/* All went well.. */
@@ -626,17 +486,29 @@ static void
 gpilot_network_device_free (GPilotDevice *device)
 {
 	if (device->fd) {
-		close (device->fd);
+		pi_close (device->fd);
 	}
-
-	g_free(device->ip);
-	g_free(device->host);
-	g_free(device->netmask);
+	if (device->ip != NULL)
+		g_free(device->ip);
 }
 
 static void
 gpilot_network_device_deinit (GPilotDevice *device)
 {
+	GError *err = NULL;
+
+	if (device->io) {
+		g_source_remove (device->in_handle);
+		g_source_remove (device->err_handle);
+		g_io_channel_shutdown (device->io, 0, &err);
+		if (err != NULL)
+		{
+			/* Report error to user, and free error */
+			g_warning ("error from shutdown: %s\n", err->message);
+			g_error_free (err);
+		} 
+		g_io_channel_unref (device->io);
+	}
 }
 #endif /* WITH_NETWORK */
 
@@ -660,6 +532,8 @@ gpilot_device_init (GPilotDevice *device)
 #ifdef WITH_NETWORK
 	} else if (device->type == PILOT_DEVICE_NETWORK) {
 		result = gpilot_network_device_init (device);
+	} else if (device->type == PILOT_DEVICE_BLUETOOTH) {
+		result = gpilot_network_device_init (device);
 #endif /* WITH_NETWORK */
 	} else {
 		g_warning (_("Unknown device type"));
@@ -680,8 +554,6 @@ static void
 gpilot_network_device_load (GPilotDevice *device)
 {
 	device->ip = gnome_config_get_string ("ip");
-	device->host = gnome_config_get_string ("host");
-	device->netmask = gnome_config_get_string ("netmask");
 }
 
 gint
@@ -712,6 +584,8 @@ gpilot_device_load (GPilotDevice *device, gint i)
 	case PILOT_DEVICE_NETWORK:
 		gpilot_network_device_load (device);
 		break;
+	case PILOT_DEVICE_BLUETOOTH:
+		break; /* no further info required */
 	default:
 		g_warning (_("Unknown device type"));
 	}
@@ -739,6 +613,8 @@ gpilot_device_deinit (GPilotDevice *device)
 #ifdef WITH_NETWORK
 	} else if (device->type == PILOT_DEVICE_NETWORK) {
 		gpilot_network_device_deinit (device);
+	} else if (device->type == PILOT_DEVICE_BLUETOOTH) {
+		gpilot_network_device_deinit (device);
 #endif /* WITH_NETWORK */
 	} else {
 		g_warning (_("Unknown device type"));
@@ -765,6 +641,8 @@ gpilot_device_free (GPilotDevice *device)
 #ifdef WITH_NETWORK
 	} else if (device->type == PILOT_DEVICE_NETWORK) {
 		gpilot_network_device_free (device);
+	} else if (device->type == PILOT_DEVICE_BLUETOOTH) {
+		gpilot_network_device_free (device);
 #endif /* WITH_NETWORK */
 	} else {
 		g_warning (_("Unknown device type"));
@@ -784,7 +662,7 @@ get_devices (void)
 	if (n==0) {
 		g_warning (_("Number of devices is configured to 0"));
 #ifdef GUI
-		gpilot_gui_warning_dialog (_("No cradles configure.\n"
+		gpilot_gui_warning_dialog (_("No devices configured.\n"
 					    "Please run gpilotd-control-applet (use gnomecc)\n" 
 					    "to configure gnome-pilot."));    
 #endif
@@ -820,9 +698,9 @@ get_pilots (void)
 	n = gnome_config_get_int ("/gnome-pilot.d/gpilotd/General/num_pilots=0");
 
 	if (n==0) {
-		g_warning (_("Number of pilots is configured to 0"));
+		g_warning (_("Number of PDAs is configured to 0"));
 #ifdef GUI
-		gpilot_gui_warning_dialog (_("No pilots configured.\n"
+		gpilot_gui_warning_dialog (_("No PDAs configured.\n"
 					    "Please run gpilotd-control-applet (use gnomecc)\n" 
 					    "to configure gnome-pilot."));    
 #endif
@@ -852,15 +730,13 @@ void
 gpilot_pilot_init (GPilotPilot *pilot, 
 		   gint i)
 {
-	gchar *tmp_str;
 	gchar *prefix;
 
 	/* set up stuff  */
 	g_free (pilot->name);
 	g_free (pilot->passwd);
 	g_free (pilot->pilot_username);
-	g_list_foreach (pilot->trusted_users, (GFunc) g_free, NULL);
-	g_list_free (pilot->trusted_users);
+	g_free (pilot->pilot_charset);
 	prefix = g_strdup_printf ("/gnome-pilot.d/gpilotd/Pilot%d/", i);
 	gnome_config_push_prefix (prefix);
 
@@ -872,11 +748,29 @@ gpilot_pilot_init (GPilotPilot *pilot,
 	pilot->creation = gnome_config_get_int ("creation");
 	pilot->romversion = gnome_config_get_int ("romversion");
 	pilot->number=i;
-	pilot->trusted_users=NULL;
-	tmp_str = gnome_config_get_string("default_sync_action=synchronize");
-	pilot->sync_options.default_sync_action = gnome_pilot_conduit_sync_type_str_to_int(tmp_str);
-	g_free (tmp_str);
 	pilot->sync_options.basedir = gnome_config_get_string("basedir");
+#ifdef PILOT_LINK_0_12
+	pilot->pilot_charset = gnome_config_get_string("charset");
+	/* If no charset has been specified by user, fall back
+	 * to the PILOT_CHARSET environment variable, if that
+	 * has been specified.
+	 */
+	if (pilot->pilot_charset == NULL) {
+		if ((pilot->pilot_charset
+			= getenv("PILOT_CHARSET")) != NULL) {
+			pilot->pilot_charset =
+			    g_strdup (pilot->pilot_charset);
+		} else {
+			g_warning (_("No pilot_charset specified.  Using `%s'."),
+			    GPILOT_DEFAULT_CHARSET);
+			pilot->pilot_charset =
+			    g_strdup (GPILOT_DEFAULT_CHARSET);
+		}
+	}
+#else
+	pilot->pilot_charset=NULL;
+#endif
+
 
 	gnome_config_pop_prefix ();
 	g_free(prefix);
@@ -889,8 +783,7 @@ gpilot_pilot_free (GPilotPilot *pilot)
 	g_free (pilot->passwd);
 	g_free (pilot->pilot_username);
 	g_free (pilot->sync_options.basedir);
-	g_list_foreach (pilot->trusted_users, (GFunc) g_free, NULL);
-	g_list_free (pilot->trusted_users);
+	g_free (pilot->pilot_charset);
 	g_free (pilot);
 }
 
@@ -944,19 +837,19 @@ gnome_pilot_conduit_sync_type_str_to_int(const gchar *s)
 {
 	g_return_val_if_fail(s!=NULL,GnomePilotConduitSyncTypeNotSet);
 
-	if (strcmp (s, "synchronize") == 0) {
+	if (strcmp (s, GnomePilotConduitSyncTypeSynchronizeStr) == 0) {
 		return GnomePilotConduitSyncTypeSynchronize;
-	} else if (strcmp (s, "copy_to_pilot") == 0) {
+	} else if (strcmp (s, GnomePilotConduitSyncTypeCopyToPilotStr) == 0) {
 		return GnomePilotConduitSyncTypeCopyToPilot;
-	} else if (strcmp (s, "copy_from_pilot") == 0) {
+	} else if (strcmp (s, GnomePilotConduitSyncTypeCopyFromPilotStr) == 0) {
 		return GnomePilotConduitSyncTypeCopyFromPilot;
-	} else if (strcmp (s, "merge_to_pilot") == 0) {
+	} else if (strcmp (s, GnomePilotConduitSyncTypeMergeToPilotStr) == 0) {
 		return GnomePilotConduitSyncTypeMergeToPilot;
-	} else if (strcmp (s, "merge_from_pilot") == 0) {
+	} else if (strcmp (s, GnomePilotConduitSyncTypeMergeFromPilotStr) == 0) {
 		return GnomePilotConduitSyncTypeMergeFromPilot;
-	} else if (strcmp (s, "custom") == 0) {
+	} else if (strcmp (s, GnomePilotConduitSyncTypeCustomStr) == 0) {
 		return GnomePilotConduitSyncTypeCustom;
-	} else if (strcmp (s, "not_set") == 0) {
+	} else if (strcmp (s, GnomePilotConduitSyncTypeNotSetStr) == 0) {
 		return GnomePilotConduitSyncTypeNotSet;
 	}
 	return GnomePilotConduitSyncTypeNotSet;
@@ -966,15 +859,22 @@ const gchar*
 gnome_pilot_conduit_sync_type_int_to_str(GnomePilotConduitSyncType e) 
 {
 	switch(e) {
-	case GnomePilotConduitSyncTypeCustom:         return "custom";
-	case GnomePilotConduitSyncTypeSynchronize:    return "synchronize";
-	case GnomePilotConduitSyncTypeCopyFromPilot:  return "copy_from_pilot";
-	case GnomePilotConduitSyncTypeCopyToPilot:    return "copy_to_pilot";
-	case GnomePilotConduitSyncTypeMergeFromPilot: return "merge_from_pilot";
-	case GnomePilotConduitSyncTypeMergeToPilot:   return "merge_to_pilot";
-	case GnomePilotConduitSyncTypeNotSet:         return "not_set";
+	case GnomePilotConduitSyncTypeCustom:
+		return GnomePilotConduitSyncTypeCustomStr;
+	case GnomePilotConduitSyncTypeSynchronize:
+		return GnomePilotConduitSyncTypeSynchronizeStr;
+	case GnomePilotConduitSyncTypeCopyFromPilot:
+		return GnomePilotConduitSyncTypeCopyFromPilotStr;
+	case GnomePilotConduitSyncTypeCopyToPilot:
+		return GnomePilotConduitSyncTypeCopyToPilotStr;
+	case GnomePilotConduitSyncTypeMergeFromPilot:
+		return GnomePilotConduitSyncTypeMergeFromPilotStr;
+	case GnomePilotConduitSyncTypeMergeToPilot:
+		return GnomePilotConduitSyncTypeMergeToPilotStr;
+	case GnomePilotConduitSyncTypeNotSet:
+		return GnomePilotConduitSyncTypeNotSetStr;
 	default: 
 	  g_message ("gnome_pilot_conduit_sync_type_int_to_str: invalid sync_type %d",e);
-	  return "not_set";
+	  return GnomePilotConduitSyncTypeNotSetStr;
 	}
 }
