@@ -74,7 +74,7 @@
 #include "gpilot-gui.h"
 #include "gnome-pilot-conduit-backup.h"
 
-#include <libgpilotdCM/gnome-pilot-conduit-management.h>
+#include "../libgpilotdCM/gnome-pilot-conduit-management.h"
 
 static GPilotContext *context = NULL;
 
@@ -1079,7 +1079,7 @@ static void
 hal_device_added (LibHalContext *ctx, const char *udi)
 {
 	gboolean visor_net = FALSE;
-	char *bus, *match_str;
+	char *bus, *platform, *match_str;
 	int vendor_id, product_id, i;
 	GPilotDevice *device;
 	DBusError error;
@@ -1090,15 +1090,25 @@ hal_device_added (LibHalContext *ctx, const char *udi)
 
 	load_devices_xml ();
 
-	/* HAL match rule: we look for info.bus == 'usb_device'
+	/* HAL match rule: we look for pda.platform == 'palm'
+	 * (or the legacy info.bus == 'usb_device')
 	 * and then try to match the usb_device.product_id and usb_device.vendor_id
 	 * against the list in devices.xml.
 	 */
-	if (!(bus = libhal_device_get_property_string (hal_ctx, udi, "info.bus", NULL)))
+	if (platform = libhal_device_get_property_string (hal_ctx, udi, "pda.platform", NULL)) {
+	    if (strcmp (platform, "palm") != 0) {
+		libhal_free_string (platform);
 		return;
-	if (strcmp (bus, "usb_device") != 0) {
+	    }
+	    libhal_free_string (platform);
+	} else if (bus = libhal_device_get_property_string (hal_ctx, udi, "info.bus", NULL)) {
+	    if (strcmp (bus, "usb_device") != 0) {
 		libhal_free_string (bus);
 		return;
+	    }
+	    libhal_free_string (bus);
+	} else {
+	    return;
 	}
 
 	dbus_error_init (&error);
