@@ -45,8 +45,8 @@
 #include "gnome-pilot-dbinfo.h"
 #include "gpilot-gui.h"
 
-#include <libgpilotdCM/gnome-pilot-conduit-management.h>
-#include <libgpilotdCM/gnome-pilot-conduit-config.h>
+#include "../libgpilotdCM/gnome-pilot-conduit-management.h"
+#include "../libgpilotdCM/gnome-pilot-conduit-config.h"
 
 #define GPCM_LABEL "conduit_mgmt"
 #define GPCC_LABEL "conduit_cfg"
