@@ -46,7 +46,7 @@
 #define ENT_HANDLE "handle"
 #define ENT_CONT_SYNC "continue_sync"
 
-static gint is_system_related(GPilotRequestType type) {
+static gint is_system_related (GPilotRequestType type) {
 	switch(type) {
 	case GREQ_CRADLE_EVENT:
 	case GREQ_SET_USERINFO:
@@ -64,32 +64,38 @@ static gint is_system_related(GPilotRequestType type) {
   crapcrap, set_section skal tage **, man skal ikke malloc den... duhduh!
 */
 
-static void set_section(guint32 pilot_id,GPilotRequestType type,gchar **section) {
+static void set_section (guint32 pilot_id, GPilotRequestType type, gchar **section) {
 	g_assert(section!=NULL);
 	if (*section!=NULL) { g_warning("set_section: *section!=NULL, potiential leak!"); }
 
-	if(!is_system_related(type))
-		(*section) = g_strdup_printf("%s%u/",QUEUE,pilot_id);
-	else
-		(*section) = g_strdup_printf("%ssystem/",QUEUE);
+	if(!is_system_related(type)) {
+		(*section) = g_strdup_printf ("%s%u/", QUEUE, pilot_id);
+	} else {
+		(*section) = g_strdup_printf ("%ssystem/", QUEUE);
+	}
 }
 
-static guint32 set_section_num(guint32 pilot_id,GPilotRequestType type,
-			       gchar **section,gint num) {
-	g_assert(section!=NULL);
+static guint32 set_section_num (guint32 pilot_id,
+				GPilotRequestType type,
+				gchar **section,
+				gint num) {
+	g_assert (section!=NULL);
 	if (*section!=NULL) { g_warning("set_section_num: *section!=NULL, potiential leak!"); }
 
-	if(!is_system_related(type)) {
-		(*section) = g_strdup_printf("%s%u-%u/",QUEUE,pilot_id,num);
+	if(!is_system_related (type)) {
+		(*section) = g_strdup_printf ("%s%u-%u/", QUEUE, pilot_id, num);
 		return pilot_id*65535+num;
 	} else {
-		(*section) = g_strdup_printf("%ssystem-%d/",QUEUE,num);
+		(*section) = g_strdup_printf ("%ssystem-%d/", QUEUE, num);
 		return num;
 	}
 }
 
 static GPilotRequestType request_type_from_string(gchar *str) {
-	g_return_val_if_fail(str!=NULL,GREQ_INVALID);
+	if (str==NULL) {
+		g_warning (_("Error in queue, non-existing entry"));
+		return GREQ_INVALID;
+	}
 
 	if(g_strcasecmp(str,"GREQ_INSTALL")==0) return GREQ_INSTALL;
 	if(g_strcasecmp(str,"GREQ_RESTORE")==0) return GREQ_RESTORE;
@@ -110,7 +116,7 @@ GList* gpc_queue_load_requests_for_cradle(gchar *cradle) {
 	tmp = NULL;
 	it = NULL;
 	/* call with a system type request and all=TRUE, to get all the queued system requests */
-	tmp = gpc_queue_load_requests(0,GREQ_CRADLE_EVENT,TRUE);
+	tmp = gpc_queue_load_requests (0, GREQ_CRADLE_EVENT, TRUE);
 
 	it = g_list_first(tmp);
 	while(it) {
@@ -127,68 +133,76 @@ GList* gpc_queue_load_requests_for_cradle(gchar *cradle) {
 	g_list_free(tmp);
 
 	return retval;
-};
+}
 
-GList* gpc_queue_load_requests(guint32 pilot_id,GPilotRequestType type,gboolean all) {
+GList* gpc_queue_load_requests (guint32 pilot_id, GPilotRequestType type, gboolean all) {
 	int num;
 	GList *retval = NULL;
 	gchar *prefix = NULL;
 
-	set_section(pilot_id,type,&prefix);
+	set_section (pilot_id, type, &prefix);
  
-	gnome_config_push_prefix(prefix);
-	num = gnome_config_get_int(NUMREQ);
-	gnome_config_pop_prefix();
-	g_free(prefix);
+	gnome_config_push_prefix (prefix);
+	num = gnome_config_get_int (NUMREQ);
+	gnome_config_pop_prefix ();
+	g_free (prefix);
 
-	for(;num>0;num--) {
+	for (;num>0;num--) {
 		GPilotRequest *req;
-		if(is_system_related(type)) 
-			req = gpc_queue_load_request(pilot_id,TRUE,num);
-		else
-			req = gpc_queue_load_request(pilot_id,FALSE,num);
+		if (is_system_related (type)) {
+			req = gpc_queue_load_request (pilot_id, TRUE, num);
+		} else {
+			req = gpc_queue_load_request (pilot_id, FALSE, num);
+		}
+
 		if(req==NULL) {
-			gnome_config_pop_prefix();
+			gnome_config_pop_prefix ();
 			continue;
 		}
-		if(req->type!=type && !all) {
-			g_free(req);
-			gnome_config_pop_prefix();
+		if (req->type!=type && all==FALSE) {
+			g_free (req);
+			gnome_config_pop_prefix ();
 			continue;
 		}
-		retval = g_list_append(retval,req);
-		gnome_config_pop_prefix();
+
+		retval = g_list_append (retval, req);
+		gnome_config_pop_prefix ();
 	}
 	return retval;
 }
 
-GPilotRequest* gpc_queue_load_request(guint32 pilot_id,gboolean _type,guint num) {
+GPilotRequest* 
+gpc_queue_load_request (guint32 pilot_id, gboolean _type, guint num) {
 	GPilotRequest *req;
 	gchar *prefix = NULL;
 	GPilotRequestType type;
   
-	if(_type==TRUE) type = GREQ_CRADLE_EVENT; else type = GREQ_PILOT_EVENT;
+	if (_type==TRUE) {
+		type = GREQ_CRADLE_EVENT; 
+	} else {
+		type = GREQ_PILOT_EVENT;
+	}
 
-	set_section_num(pilot_id,type,&prefix,num);
-	gnome_config_push_prefix(prefix);
+	set_section_num (pilot_id, type, &prefix, num);
+	gnome_config_push_prefix (prefix);
 
-	req = g_new0(GPilotRequest,sizeof(GPilotRequest));
-	req->type = request_type_from_string(gnome_config_get_string(ENT_TYPE));
-	if(req->type==GREQ_INVALID) {
-		g_free(req);
-		gnome_config_pop_prefix();
+	req = g_new0 (GPilotRequest, sizeof (GPilotRequest));
+	req->type = request_type_from_string (gnome_config_get_string (ENT_TYPE));
+	if (req->type==GREQ_INVALID) {
+		g_free (req);
+		gnome_config_pop_prefix ();
 		return NULL;
 	}
   
 	/* unless I store the sectionname _without_ trailing /, clean_section
 	   can't delete it ? */
-	g_free(prefix);
+	g_free (prefix);
 	prefix = NULL;
-	req->handle = set_section_num(pilot_id,type,&prefix,num);
-	req->queue_data.section_name = g_strdup(prefix);
+	req->handle = set_section_num (pilot_id, type, &prefix, num);
+	req->queue_data.section_name = g_strdup (prefix);
 	req->pilot_id = pilot_id;
   
-	switch(req->type) {
+	switch (req->type) {
 	case GREQ_INSTALL:
 		req->parameters.install.filename = gnome_config_get_string(ENT_FILENAME);
 		req->parameters.install.description = gnome_config_get_string(ENT_DESCRIPTION);
@@ -307,7 +321,7 @@ guint gpc_queue_store_request(GPilotRequest req) {
 	gnome_config_sync();
 	g_free(prefix);
 
-	LOG("assigned handle num %u",handle_num);
+	LOG(("assigned handle num %u",handle_num));
 	return handle_num;
 }
 
@@ -351,7 +365,7 @@ gpc_queue_purge_request(GPilotRequest **req)
 	gchar *prefix = NULL;
 	int num;
 
-	LOG("gpc_queue_purge_request()");
+	LOG(("gpc_queue_purge_request()"));
 
 	g_return_if_fail(req!=NULL);
 	g_return_if_fail(*req!=NULL);
@@ -400,18 +414,25 @@ gpc_queue_purge_request(GPilotRequest **req)
 } 
 
 void 
-gpc_queue_purge_request_point(guint32 pilot_id,guint num) 
+gpc_queue_purge_request_point (guint32 pilot_id,
+			       guint num) 
 {
 	GPilotRequest *req;
 
-	if(pilot_id==0) /* no id, system level request, use system level request
-			   type for load */
-		req = gpc_queue_load_request(pilot_id,TRUE,num);
-	else /* otherwise use a common as type */
-		req = gpc_queue_load_request(pilot_id,FALSE,num);
+	if(pilot_id==0) {
+                 /* no id, system level request, use system level request
+		    type for load */
+		g_message ("FISK: OST");
+		req = gpc_queue_load_request (pilot_id, TRUE, num);
+	} else {
+                /* otherwise use a common as type */
+		g_message ("FISK: KRYDDERSILD");
+		req = gpc_queue_load_request (pilot_id, FALSE, num);
+	}
 
-	if(req)
-		gpc_queue_purge_request(&req);  
-	else
-		g_warning(_("fault: no request found for pilot %d, request %d"),pilot_id,num);
+	if(req) {
+		gpc_queue_purge_request (&req);  
+	} else {
+		g_warning (_("fault: no request found for PDA %d, request %d"), pilot_id, num);
+	}
 }
