@@ -26,7 +26,7 @@ static const struct poptOption options[] = {
 	{"listconduits", 'l', POPT_ARG_NONE, &arg_listconduits, 0, N_("lists available conduits"), NULL},
 	{"conduit", '\0', POPT_ARG_STRING, &arg_conduit, 0, N_("conduit to operate on"), N_("CONDUIT")},
 	{"listallattribs", 'A', POPT_ARG_NONE, &arg_allattrs, 0, N_("list all attributes"), NULL},
-	{"pilotid", '\0', POPT_ARG_INT, &arg_pilot, 0, N_("specify pilot to operate on"), N_("PILOTID")},
+	{"pilotid", '\0', POPT_ARG_INT, &arg_pilot, 0, N_("specify PDA to operate on"), N_("PDA_ID")},
 	{"listattribs",'a', POPT_ARG_NONE, &arg_attrs, 0, N_("list attributes"), NULL},
 	{"listconduitswithstat",'s', POPT_ARG_NONE, &arg_listconduitsstate, 0, N_("list available conduits and their state"), NULL},
 	{"enable",'e', POPT_ARG_NONE, &arg_enable, 0, N_("enable specified conduit"), NULL},
@@ -99,6 +99,7 @@ main (int argc, char *argv[]) {
 	
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	gnome_init_with_popt_table ("gpilotdcm-client","1.0", argc, argv, options,0,&pctx);
 
@@ -107,7 +108,7 @@ main (int argc, char *argv[]) {
 	gnome_pilot_conduit_management_get_conduits (&conduits, GNOME_PILOT_CONDUIT_MGMT_ID);
 	
 	ll = gnome_i18n_get_language_list (NULL);
-	g_message ("lang = %s", ll->data);
+	g_message ("lang = %s", (char *) ll->data);
 
 	/* default to list all conduit attributes,
 	   unless a pilot id is given, then dump status */
